var KeyValueEntry = /** @class */ (function () {
    function KeyValueEntry(type) {
        this._value = undefined;
        this._type = type;
    }
    Object.defineProperty(KeyValueEntry.prototype, "type", {
        get: function () {
            return this._type;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "id", {
        get: function () {
            return this._type.id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "label", {
        get: function () {
            return this._type.label;
        },
        enumerable: true,
        configurable: true
    });
    KeyValueEntry.prototype.getSuggestions = function (prefix) {
        return this._type.getSuggestions(prefix);
    };
    KeyValueEntry.prototype.parse = function (val) {
        if (typeof val === "string") {
            return this._type.parseString(val);
        }
        else if (val !== undefined) {
            return this._type.checkValue(val);
        }
        return undefined;
    };
    KeyValueEntry.prototype.set = function (val) {
        this._value = this.parse(val);
        return this._value !== undefined;
    };
    Object.defineProperty(KeyValueEntry.prototype, "value", {
        get: function () {
            return this._value;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "isPartial", {
        get: function () {
            return this.value === undefined;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "display", {
        get: function () {
            if (this.value === undefined) {
                return undefined;
            }
            return this._type.display(this.value);
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueEntry.prototype, "editText", {
        get: function () {
            if (this.value === undefined) {
                return undefined;
            }
            return this._type.editText(this.value);
        },
        enumerable: true,
        configurable: true
    });
    KeyValueEntry.prototype.equals = function (entry) {
        return this._type.isEqual(this, entry);
    };
    return KeyValueEntry;
}());
export { KeyValueEntry };
var KeyValueType = /** @class */ (function () {
    function KeyValueType(id, label, aliases) {
        if (aliases === void 0) { aliases = []; }
        this._id = id;
        this._label = label;
        this._aliases = [id, label].concat(aliases).map(function (a) { return a.toLowerCase(); });
    }
    Object.defineProperty(KeyValueType.prototype, "id", {
        get: function () {
            return this._id;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueType.prototype, "label", {
        get: function () {
            return this._label;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(KeyValueType.prototype, "aliases", {
        get: function () {
            return this._aliases;
        },
        enumerable: true,
        configurable: true
    });
    KeyValueType.prototype.findAlias = function (substring) {
        var substringLower = substring.toLowerCase();
        return this.aliases.find(function (a) { return a.includes(substringLower); });
    };
    KeyValueType.prototype.createPartialEntry = function () {
        return new KeyValueEntry(this);
    };
    KeyValueType.prototype.tryCreatePartialEntry = function (key) {
        if (!this.aliases.includes(key.toLowerCase())) {
            return undefined;
        }
        return new KeyValueEntry(this);
    };
    KeyValueType.prototype.tryCreateFullEntry = function (val) {
        var entry = new KeyValueEntry(this);
        if (entry.set(val)) {
            return entry;
        }
        else {
            return undefined;
        }
    };
    KeyValueType.prototype.isEqual = function (entryA, entryB) {
        return entryA.type === entryB.type && entryA.value === entryB.value;
    };
    return KeyValueType;
}());
export { KeyValueType };
